"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.afterEach = exports.beforeEach = void 0;
const each_js_1 = require("../../configs/each.js");
/**
 * Handle **global states** and **external** services before each `test` or `it`.
 *
 * ---
 *
 * ```ts
 * import { beforeEach } from 'poku';
 *
 * const before = beforeEach(() => {
 *   // prepare
 * };
 *
 * before.pause();
 * before.continue();
 * before.reset();
 * ```
 */
const beforeEach = (callback, options) => {
    each_js_1.each.before.test = typeof (options === null || options === void 0 ? void 0 : options.test) === 'boolean' ? options.test : true;
    each_js_1.each.before.assert =
        typeof (options === null || options === void 0 ? void 0 : options.assert) === 'boolean' ? options.assert : false;
    (options === null || options === void 0 ? void 0 : options.immediate) && callback();
    each_js_1.each.before.cb = () => {
        if (each_js_1.each.before.status) {
            callback();
        }
    };
    const pause = () => {
        each_js_1.each.before.status = false;
    };
    const continueFunc = () => {
        each_js_1.each.before.status = true;
    };
    const reset = () => {
        each_js_1.each.before.cb = undefined;
    };
    return { pause, continue: continueFunc, reset };
};
exports.beforeEach = beforeEach;
/**
 * Handle **global states** and **external** services after each `test` or `it`.
 *
 * ---
 *
 * ```ts
 * import { afterEach } from 'poku';
 *
 * const after = afterEach(() => {
 *   // cleanup
 * };
 *
 * after.pause();
 * after.continue();
 * after.reset();
 * ```
 */
/* c8 ignore next */ // ?
const afterEach = (callback, options) => {
    each_js_1.each.after.test = typeof (options === null || options === void 0 ? void 0 : options.test) === 'boolean' ? options.test : true;
    each_js_1.each.after.assert =
        typeof (options === null || options === void 0 ? void 0 : options.assert) === 'boolean' ? options.assert : false;
    each_js_1.each.after.cb = () => {
        if (each_js_1.each.after.status) {
            callback();
        }
    };
    const pause = () => {
        each_js_1.each.after.status = false;
    };
    const continueFunc = () => {
        each_js_1.each.after.status = true;
    };
    const reset = () => {
        each_js_1.each.after.cb = undefined;
    };
    return { pause, continue: continueFunc, reset };
};
exports.afterEach = afterEach;
