"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.kill = void 0;
const get_runner_js_1 = require("../../parsers/get-runner.js");
const pid_js_1 = require("../../services/pid.js");
const get_pids_js_1 = require("./get-pids.js");
const killPID = (PID) => __awaiter(void 0, void 0, void 0, function* () {
    const PIDs = (0, pid_js_1.setPortsAndPIDs)(PID);
    yield Promise.all(PIDs.map((p) => __awaiter(void 0, void 0, void 0, function* () {
        get_runner_js_1.isWindows
            ? yield pid_js_1.killPID.windows(p)
            : yield pid_js_1.killPID.unix(p);
    })));
});
const killPort = (port) => __awaiter(void 0, void 0, void 0, function* () {
    const PIDs = yield (0, get_pids_js_1.getPIDs)(port);
    for (const PID of PIDs) {
        if (!PID) {
            continue;
        }
        yield killPID(PID);
    }
});
const killRange = (startsAt, endsAt) => __awaiter(void 0, void 0, void 0, function* () {
    const PIDs = yield get_pids_js_1.getPIDs.range(startsAt, endsAt);
    for (const PID of PIDs) {
        if (!PID) {
            continue;
        }
        yield killPID(PID);
    }
});
/** Kill processes by PIDs, ports and port ranges. */
exports.kill = {
    /** Kill the specified process ID */
    pid: killPID,
    /** Kill all processes listening on the specified port */
    port: killPort,
    /** Kill all processes listening on the specified range ports */
    range: killRange,
};
