"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.listFiles = exports.getAllFiles = exports.escapeRegExp = exports.isFile = exports.sanitizePath = void 0;
const node_process_1 = require("process");
const node_path_1 = require("path");
const fs_js_1 = require("../../polyfills/fs.js");
const regex = {
    sep: /[/\\]+/g,
    pathLevel: /(\.\.(\/|\\|$))+/g,
    unusualChars: /[<>|^?*]+/g,
    absolutePath: /^[/\\]/,
    safeRegExp: /[.*{}[\]\\]/g,
    defaultFilter: /\.(test|spec)\./i,
};
const sanitizePath = (input, ensureTarget) => {
    const sanitizedPath = input
        .replace(regex.sep, node_path_1.sep)
        .replace(regex.pathLevel, '')
        .replace(regex.unusualChars, '');
    return ensureTarget
        ? sanitizedPath.replace(regex.absolutePath, `.${node_path_1.sep}`)
        : sanitizedPath;
};
exports.sanitizePath = sanitizePath;
const isFile = (fullPath) => __awaiter(void 0, void 0, void 0, function* () { return (yield (0, fs_js_1.stat)(fullPath)).isFile(); });
exports.isFile = isFile;
const escapeRegExp = (string) => string.replace(regex.safeRegExp, '\\$&');
exports.escapeRegExp = escapeRegExp;
const envFilter = ((_a = node_process_1.env.FILTER) === null || _a === void 0 ? void 0 : _a.trim())
    ? new RegExp((0, exports.escapeRegExp)(node_process_1.env.FILTER), 'i')
    : undefined;
const getAllFiles = (dirPath_1, ...args_1) => __awaiter(void 0, [dirPath_1, ...args_1], void 0, function* (dirPath, files = new Set(), configs) {
    const currentFiles = yield (0, fs_js_1.readdir)((0, exports.sanitizePath)(dirPath));
    const filter = envFilter
        ? envFilter
        : (configs === null || configs === void 0 ? void 0 : configs.filter) instanceof RegExp
            ? configs.filter
            : regex.defaultFilter;
    const exclude = (configs === null || configs === void 0 ? void 0 : configs.exclude)
        ? Array.isArray(configs.exclude)
            ? configs.exclude
            : [configs.exclude]
        : undefined;
    yield Promise.all(currentFiles.map((file) => __awaiter(void 0, void 0, void 0, function* () {
        const fullPath = (0, node_path_1.join)(dirPath, file);
        const stat = yield (0, fs_js_1.stat)(fullPath);
        /* c8 ignore next 6 */
        if (fullPath.indexOf('node_modules') !== -1 ||
            fullPath.indexOf('.git') === 0) {
            return;
        }
        if (exclude) {
            for (const pattern of exclude) {
                if (pattern.test(fullPath)) {
                    return;
                }
            }
        }
        if (filter.test(fullPath)) {
            return files.add(fullPath);
        }
        if (stat.isDirectory()) {
            yield (0, exports.getAllFiles)(fullPath, files, configs);
        }
    })));
    return files;
});
exports.getAllFiles = getAllFiles;
/* c8 ignore next */ // ?
const listFiles = (targetDir, configs) => __awaiter(void 0, void 0, void 0, function* () { return Array.from(yield (0, exports.getAllFiles)((0, exports.sanitizePath)(targetDir), new Set(), configs)); });
exports.listFiles = listFiles;
