"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.argToArray = exports.getLastParam = exports.hasArg = exports.getArg = void 0;
/* c8 ignore next */ // ?
const node_process_1 = require("process");
const [, , ...processArgs] = node_process_1.argv;
const regexQuotes = /''|""/;
/**
 * Gets the value of an argument.
 *
 * ---
 *
 * CLI arguments examples:
 *
 * ```sh
 * command --arg=some # 'some'
 * command --arg=""   # ''
 * command --arg      # undefined
 * ```
 */
const getArg = (arg, prefix = '--', baseArgs = processArgs) => {
    const argPattern = `${prefix}${arg}=`;
    const argValue = baseArgs.find((a) => a.startsWith(argPattern));
    if (!argValue) {
        return undefined;
    }
    return argValue.slice(argPattern.length).replace(regexQuotes, '');
};
exports.getArg = getArg;
/**
 * Checks if an argument exists.
 *
 * ---
 *
 * CLI arguments examples:
 *
 * ```sh
 * command --arg  # true
 * command        # false
 * ```
 */
const hasArg = (arg, prefix = '--', baseArgs = processArgs) => {
    const argPattern = `${prefix}${arg}`;
    return baseArgs.some((a) => a.startsWith(argPattern));
};
exports.hasArg = hasArg;
/**
 * Gets the last param/value.
 *
 * CLI arguments examples:
 *
 * ```sh
 * command --arg --arg2=some value  # 'value'
 * command value                    # 'value'
 * command                          # undefined
 * command --arg                    # undefined
 * ```
 */
const getLastParam = (prefix = '--', baseArgs = processArgs) => {
    const lastArg = baseArgs[baseArgs.length - 1];
    if (!lastArg || lastArg.startsWith(prefix)) {
        return undefined;
    }
    return lastArg;
};
exports.getLastParam = getLastParam;
/* c8 ignore next */ // ?
const argToArray = (arg, prefix = '--', baseArgs = processArgs) => {
    const hasArgument = (0, exports.hasArg)(arg, prefix, baseArgs);
    if (!hasArgument) {
        return undefined;
    }
    const argValue = (0, exports.getArg)(arg, prefix, baseArgs);
    if (!argValue) {
        return [];
    }
    return argValue
        .split(',')
        .map((a) => a.trim())
        .filter((a) => a);
};
exports.argToArray = argToArray;
