"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processAssert = void 0;
const node_process_1 = require("process");
const node_path_1 = __importDefault(require("path"));
const node_assert_1 = require("assert");
const find_file_from_stack_js_1 = require("../parsers/find-file-from-stack.js");
const assert_js_1 = require("../parsers/assert.js");
const each_js_1 = require("../configs/each.js");
const indentation_js_1 = require("../configs/indentation.js");
const format_js_1 = require("./format.js");
const write_js_1 = require("./write.js");
const cwd = (0, node_process_1.cwd)();
const regexFile = /file:(\/\/)?/;
const processAssert = (cb, options) => __awaiter(void 0, void 0, void 0, function* () {
    const isPoku = typeof (node_process_1.env === null || node_process_1.env === void 0 ? void 0 : node_process_1.env.FILE) === 'string' && (node_process_1.env === null || node_process_1.env === void 0 ? void 0 : node_process_1.env.FILE.length) > 0;
    const FILE = node_process_1.env.FILE;
    let preIdentation = '';
    if (indentation_js_1.indentation.hasDescribe || indentation_js_1.indentation.hasTest) {
        preIdentation += '  ';
    }
    if (indentation_js_1.indentation.hasIt) {
        preIdentation += '  ';
    }
    try {
        if (typeof each_js_1.each.before.cb === 'function' && each_js_1.each.before.assert) {
            const beforeResult = each_js_1.each.before.cb();
            /* c8 ignore next 3 */
            if (beforeResult instanceof Promise) {
                yield beforeResult;
            }
        }
        const cbResult = cb();
        /* c8 ignore next 3 */
        if (cbResult instanceof Promise) {
            yield cbResult;
        }
        if (typeof each_js_1.each.after.cb === 'function' && each_js_1.each.after.assert) {
            const afterResult = each_js_1.each.after.cb();
            /* c8 ignore next 3 */
            if (afterResult instanceof Promise) {
                yield afterResult;
            }
        }
        if (typeof options.message === 'string') {
            const message = isPoku &&
                !indentation_js_1.indentation.hasDescribe &&
                !indentation_js_1.indentation.hasIt &&
                /* c8 ignore next 2 */
                !indentation_js_1.indentation.hasTest
                ? `${preIdentation}${(0, format_js_1.format)(`${(0, format_js_1.format)(`✔ ${options.message}`).bold()} ${(0, format_js_1.format)(`› ${FILE}`).success().dim()}`).success()}`
                : `${preIdentation}${(0, format_js_1.format)(`✔ ${options.message}`).success().bold()}`;
            write_js_1.Write.log(message);
        }
        /* c8 ignore start */
    }
    catch (error) {
        if (error instanceof node_assert_1.AssertionError) {
            const { code, actual, expected, operator } = error;
            const absoultePath = (0, find_file_from_stack_js_1.findFile)(error).replace(regexFile, '');
            const file = node_path_1.default.relative(node_path_1.default.resolve(cwd), absoultePath);
            let message = '';
            if (typeof options.message === 'string') {
                message = options.message;
            }
            else if (options.message instanceof Error) {
                message = options.message.message;
            }
            else if (typeof options.defaultMessage === 'string') {
                message = options.defaultMessage;
            }
            const finalMessage = (message === null || message === void 0 ? void 0 : message.trim().length) > 0
                ? (0, format_js_1.format)(`✘ ${message}`).fail().bold()
                : (0, format_js_1.format)('✘ No Message').fail().bold();
            write_js_1.Write.log(isPoku
                ? `${preIdentation}${finalMessage} ${(0, format_js_1.format)(`› ${FILE}`).fail().dim()}`
                : `${preIdentation}${finalMessage}`);
            file &&
                write_js_1.Write.log(`${(0, format_js_1.format)(`${preIdentation}      File`).dim()} ${file}`);
            write_js_1.Write.log(`${(0, format_js_1.format)(`${preIdentation}      Code`).dim()} ${code}`);
            write_js_1.Write.log(`${(0, format_js_1.format)(`${preIdentation}  Operator`).dim()} ${operator}\n`);
            if (!(options === null || options === void 0 ? void 0 : options.hideDiff)) {
                const splitActual = (0, assert_js_1.parseResultType)(actual).split('\n');
                const splitExpected = (0, assert_js_1.parseResultType)(expected).split('\n');
                write_js_1.Write.log((0, format_js_1.format)(`${preIdentation}  ${(options === null || options === void 0 ? void 0 : options.actual) || 'Actual'}:`).dim());
                for (const line of splitActual) {
                    write_js_1.Write.log(`${preIdentation}  ${(0, format_js_1.format)(line).fail().bold()}`);
                }
                write_js_1.Write.log(`\n${preIdentation}  ${(0, format_js_1.format)(`${(options === null || options === void 0 ? void 0 : options.expected) || 'Expected'}:`).dim()}`);
                for (const line of splitExpected) {
                    write_js_1.Write.log(`${preIdentation}  ${(0, format_js_1.format)(line).success().bold()}`);
                }
            }
            if (options.throw) {
                console.error(error);
                write_js_1.Write.hr();
            }
            (0, node_process_1.exit)(1);
        }
        // Non-assertion errors
        throw error;
    }
    /* c8 ignore stop */
});
exports.processAssert = processAssert;
